// RSLKmain.c
// Runs on rSLK2 with TM4C123 (not LF120)
// 
// Daniel and Jonathan Valvano
// Mark McDermott
// August 25, 2024
// Bump switches
//   TM4C MSPM0
//   PA5  PA27    Left, Bump 0,
//   PF0  PB15    Center Left, Bump 1,
//   PB3  PA28    Center Right, Bump 2
//   PC4  PA31    Right, Bump 3

// Motor
//   TM4C MSPM0
//   PF2  PB4  Motor_PWML, ML+, IN3, PWM M1PWM6
//   PF3  PB1  Motor_PWMR, MR+, IN1, PWM M1PWM7
//   PA3  PB0  Motor_DIR_L,ML-, IN4, GPIO 0 means forward, 1 means backward
//   PA2  PB16 Motor_DIR_R,MR-, IN2, GPIO 0 means forward, 1 means backward

// tachometer
//   TM4C MSPM0
//   PB7  PB8  ELA  T0CCP1
//   PB2  PB12 ERA  T3CCP0
//             ERB  GPIO input  (not used)
//             ELB  GPIO input  (not used)

// SSD1306 I2C OLED
//   TM4C MSPM0
//   PA6  PB2  SCL I2C clock
//   PA7  PB3  SDA I2C data

// IR analog distance sensors
//   TM4C MSPM0
//   PE4  PA26 Right  Ain9
//   PE5  PB24 Center Ain8 
//   PE1  PA24 Left   Ain2

// TF Luna TOF distance sensor
//   TM4C MSPM0
//   PB1  PA8 TxD microcontroller sensor RxD pin 2
//   PB0  PA9 RxD microcontroller sensor TxD pin 3

// TM4C123 LaunchPad pins
//   PF1 red LED 
//   PF2 blue LED 
//   PF3 green LED 
//   PF4 S2 negative logic switch
//   PF0 S1 negative logic switch (shared with Bump 1)

/* Hardware connections
  Vcc should be connected to RSLK 3.3V (not debugger)
 /------------------------------\
 |              chip      1   8 |
 | Ant                    2   7 |
 | enna       processor   3   6 |
 |                        4   5 |
 \------------------------------/ 
 Set #define  for UART2 (PD) and Reset PB5 
 
 ESP8266    TM4C123 
  1 URxD    PD7   UART out of TM4C123, 115200 baud
  2 GPIO0         +3.3V for normal operation (ground to flash)
  3 GPIO2         +3.3V
  4 GND     Gnd   GND (70mA)
  5 UTxD    PD6   UART out of ESP8266, 115200 baud
  6 Ch_PD         chip select, 10k resistor to 3.3V
  7 Reset   PB5   TM4C123 can issue output low to cause hardware reset
  8 Vcc           regulated 3.3V supply with at least 70mA
 */
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include "../inc/tm4c123gh6pm.h"
#include "../inc/LaunchPad.h"
#include "../inc/PLL.h"
#include "../inc/Bump.h"
#include "../inc/CortexM.h"
#include "../inc/Image.h"
#include "../inc/SSD1306.h"
#include "../inc/ADCSWTrigger.h"
#include "../inc/Motor.h"
#include "../inc/Timer2A.h"
#include "Tachometer.h"
#include "../inc/UART0int.h"

void TachTest(void);
void WaitForRelease(void){
  do{
    Clock_Delay1ms(5); // LaunchPad switches may bounce
  } while((LaunchPad_Input()&0x02)==0x02);
}
uint32_t Bump;
uint32_t LeftRaw,CenterRaw,RightRaw; // raw ADC samples
uint32_t Duty,Period,Change,lasts4;
#define DUMPSIZE 200
uint32_t LeftDump[DUMPSIZE],RightDump[DUMPSIZE];
uint32_t DumpI;
uint32_t LeftTach, RightTach;
int main(void){ // main2 to test bump switches
  DisableInterrupts();
  uint32_t previous;
  PLL_Init(Bus80MHz);
  LaunchPad_Init();
  UART_Init();   // test output
  Bump_Init();   // initialize four bump switches
  // note: LaunchPad switch on PF0 is shared with RSLK Bump 1 switch
  SSD1306_Init(SSD1306_SWITCHCAPVCC);

  ADC_InitTriple();
  Duty = 4;
  Period = 10000;
  Change = 1000;  
  Motor_Init();
  Motor_Forward(Duty,Duty);
  Tachometer_Init();
  Timer2A_Init(&Tachometer_CheckForStopped,800000,3); // 10ms
  EnableInterrupts();
  while(1){
//----------------------------------
    SSD1306_ClearBuffer();
    SSD1306_DrawBMP(0,60,logo,14,SSD1306_WHITE);
    SSD1306_OutBuffer();  
    SSD1306_SetCursor(0,0);
    SSD1306_OutString("Bump test");    
    SSD1306_SetCursor(0,1);
    SSD1306_OutString("Bump =");     
    UART_OutString("\n\rRSLK2 bump test");
    UART_OutString("\n\rPress SW2 (PF4) to continue to next test");
    previous = 17;
    do{
      Bump = Bump_In();
      if(Bump != previous){
        UART_OutString("\n\rBump=0x0"); UART_OutUHex(Bump);
        SSD1306_SetCursor(6,1);
        SSD1306_OutUHex7(Bump);  
      }
      previous = Bump;
      Clock_Delay1ms(5); // bump switches will bounce
    } while((LaunchPad_Input()&0x02)==0); // SW2 (PF4)
    WaitForRelease();
////----------------------------------
//    UART_OutString("\n\rRSLK2 SSD1306 test");
//    UART_OutString("\n\rPress SW2 (PF4) to continue to next test");
//    SSD1306_ClearBuffer();
//    SSD1306_DrawString(0, 0,"----- RSLK2 ---------",SSD1306_WHITE);
//    SSD1306_DrawString(0,16,"123456789012345678901",SSD1306_WHITE);
//    SSD1306_DrawString(0,24,"abcdefghijklmnopqrstu",SSD1306_WHITE);
//    SSD1306_DrawString(0,32,"ABCDEFGHIJKLMNOPQRSTU",SSD1306_WHITE);
//    SSD1306_DrawString(0,40,"!@#$%^&*()_+-=",SSD1306_WHITE);
//    SSD1306_DrawString(0,48,"123456789012345678901",SSD1306_WHITE);
//    SSD1306_DrawString(0,56,"vwxyz,./<>?;'\"[]\\{}|",SSD1306_WHITE);
//    SSD1306_OutBuffer();
//    Clock_Delay1ms(1000);  
//    SSD1306_ClearBuffer();
//    SSD1306_OutBuffer();
//    SSD1306_SetCursor(0,0);
//    SSD1306_OutString("Bump =");     
//    do{
//      Bump = Bump_In();
//      if(Bump != previous){
//        UART_OutString("\n\rBump=0x0"); UART_OutUHex(Bump);
//        SSD1306_SetCursor(6,0);
//        SSD1306_OutUHex7(Bump);       
//      }
//      previous = Bump;
//    } while((LaunchPad_Input()&0x02)==0); // SW2 (PF4)
//    WaitForRelease();
//----------------------------------
    UART_OutString("\n\rRSLK2 IR sensor test");
    UART_OutString("\n\rPress SW2 (PF4) to continue to next test");
    SSD1306_ClearBuffer();
    SSD1306_OutBuffer();
    SSD1306_SetCursor(0,0);
    SSD1306_OutString("IR sensor test");
    SSD1306_SetCursor(0,1);
    SSD1306_OutString("Left  =");
    SSD1306_SetCursor(0,2);
    SSD1306_OutString("Center=");
    SSD1306_SetCursor(0,3);
    SSD1306_OutString("Right =");
    SSD1306_SetCursor(0,4);
    SSD1306_OutString("Bump  =");  
    do{
      Bump = Bump_In();
      ADC_InTriple(&RightRaw,&CenterRaw,&LeftRaw);
      SSD1306_SetCursor(7,1);
      SSD1306_OutUDec(LeftRaw);
      SSD1306_SetCursor(7,2);
      SSD1306_OutUDec(CenterRaw);
      SSD1306_SetCursor(7,3);
      SSD1306_OutUDec(RightRaw);
      SSD1306_SetCursor(7,4);  
      SSD1306_OutUHex7(Bump);       
      UART_OutString("\n\rLeft="); UART_OutUDec(LeftRaw);
      UART_OutString(", Center="); UART_OutUDec(CenterRaw);
      UART_OutString(", Right="); UART_OutUDec(RightRaw);
      UART_OutString(", Bump=0x0"); UART_OutUHex(Bump);
      for(int i=0; i<50;i++){ // 500ms, continue on SW2 touch
        if((LaunchPad_Input()&0x02)==0x02) i=50;
        Clock_Delay1ms(10);
      }        
    } while((LaunchPad_Input()&0x02)==0x00); // SW2 (PF4)
    WaitForRelease();
//----------------------------------  
    UART_OutString("\n\rRSLK2 motor test");
    UART_OutString("\n\rPress SW2 (PF4) to continue to next test");
    SSD1306_ClearBuffer();
    SSD1306_OutBuffer();
    SSD1306_SetCursor(0,0);
    SSD1306_OutString("IR motor test");
    SSD1306_SetCursor(0,1);
    SSD1306_OutString("R rpm =");
    SSD1306_OutUDec(Period); 
    SSD1306_SetCursor(0,2);
    SSD1306_OutString("L rpm =");
    SSD1306_SetCursor(0,3);
    SSD1306_OutString("PWM(%)=");
    SSD1306_SetCursor(0,4);
    SSD1306_OutString("Bump  =");    
    Duty = 4;
    Period = 10000;
    Change = 1000;    
    lasts4 = 0;
    LeftTach=0; RightTach=0;
    int count = 0;
    do{ // 0. 10%, 20%,... 70% duty cycle
      TachTest();
      count++;
      if(count>200){ count = 0; // slow down output
        UART_OutString("\n\rLeftRPM="); UART_OutUDec(LeftTach);
        UART_OutString(", RightRPM="); UART_OutUDec(RightTach);
        UART_OutString(", Duty=");   UART_OutUDec((100*Duty)/Period);
        UART_OutString("%, Bump=0x0"); UART_OutUHex(Bump);
      }
    }while(Duty < ((3*Period)/4));
    Motor_Forward(4,4); // stop
//----------------------------------
  }
}


int main2(void){  // main2 to test/calibrate IR sensors with and OLED
  DisableInterrupts();
  PLL_Init(Bus80MHz);                   // set system clock to 80 MHz
  Bump_Init();   // initialize four bump switches
  ADC_InitTriple();
  SSD1306_Init(SSD1306_SWITCHCAPVCC);
  SSD1306_ClearBuffer();
  SSD1306_OutBuffer();
  SSD1306_SetCursor(0,0);
  SSD1306_OutString("RSLK 2");
  SSD1306_SetCursor(0,1);
  SSD1306_OutString("Left  =");
  SSD1306_SetCursor(0,2);
  SSD1306_OutString("Center=");
  SSD1306_SetCursor(0,3);
  SSD1306_OutString("Right =");
  SSD1306_SetCursor(0,4);
  SSD1306_OutString("Bump  =");
  while(1){
    Bump = Bump_In();
    ADC_InTriple(&RightRaw,&CenterRaw,&LeftRaw);
    SSD1306_SetCursor(7,1);
    SSD1306_OutUDec(LeftRaw);
    SSD1306_SetCursor(7,2);
    SSD1306_OutUDec(CenterRaw);
    SSD1306_SetCursor(7,3);
    SSD1306_OutUDec(RightRaw);
    SSD1306_SetCursor(7,4);  
    SSD1306_OutUHex7(Bump); 
  }
}
void MotorTest(void){
  uint32_t sw4;
  Bump = Bump_In();
  if(Bump){
    Duty = 4;
    Motor_Forward(Duty,Duty);  // stop
    Clock_Delay(100000); // debounce switch
    do{
      Bump = Bump_In();
      SSD1306_SetCursor(7,4);  
      SSD1306_OutUHex7(Bump); 
    }while(Bump);
    Motor_Forward(Duty,Duty);
  }
  sw4 = LaunchPad_Input();
  if(sw4 && (lasts4==0)){ // touch switch
    while(LaunchPad_Input()){
      Clock_Delay(100000); // debounce switch
    }
    sw4 = 0;
    Duty = Duty+Change;
    if(Duty >= Period){ // 10 20 30 ... 90%
      Duty = 4;
    }
    Motor_Forward(Duty,Duty);
  }
  SSD1306_SetCursor(7,2);
  SSD1306_OutUDec(Duty);
  SSD1306_SetCursor(7,3);
  SSD1306_OutUFix1((1000*Duty)/Period);
  SSD1306_SetCursor(7,4);  
  SSD1306_OutUHex7(Bump);     
  lasts4 = sw4;
}
uint32_t Duty,Period,Change;
int main3(void){  // main6 to test motors
  DisableInterrupts();

  PLL_Init(Bus80MHz);                   // set system clock to 80 MHz
  Duty = 4;
  Period = 10000;
  Change = 1000;  
  LaunchPad_Init(); // call befor Motor_Init (both access PF3 PF2)
  lasts4 = LaunchPad_Input();
  Bump_Init();   // initialize four bump switches
  SSD1306_Init(SSD1306_SWITCHCAPVCC);
  SSD1306_ClearBuffer();
  SSD1306_OutBuffer();
  SSD1306_SetCursor(0,0);
  SSD1306_OutString("RSLK 2");
  SSD1306_SetCursor(0,1);
  SSD1306_OutString("Period=");
  SSD1306_OutUDec(Period); 
  SSD1306_SetCursor(0,2);
  SSD1306_OutString("Duty  =");
  SSD1306_SetCursor(0,3);
  SSD1306_OutString("PWM(%) =");
  SSD1306_SetCursor(0,4);
  SSD1306_OutString("Bump  =");  
  SSD1306_SetCursor(7,1);
  Motor_Init();
  Motor_Forward(Duty,Duty);
  while(1){
    MotorTest();
  }  
}
void Background(void){ // used in main7 only
  Tachometer_CheckForStopped(); // if motors are stopped, sets speed to 0
  if(DumpI < DUMPSIZE){
    LeftDump[DumpI] = Tachometer_GetLeftrpm();
    RightDump[DumpI] = Tachometer_GetRightrpm();
    DumpI++;
  }
}
#define LPFSIZE 64
void TachTest(void){
  uint32_t sw4;
  Bump = Bump_In();
  if(Bump){
    Duty = 4;
    Motor_Forward(Duty,Duty);  // stop
    Clock_Delay(100000); // debounce switch
    do{
      Bump = Bump_In();
      SSD1306_SetCursor(7,4);  
      SSD1306_OutUHex7(Bump); 
    }while(Bump);
    Motor_Forward(Duty,Duty);
  }
  sw4 = LaunchPad_Input();
  if(sw4 && (lasts4==0)){ // touch switch
    while(LaunchPad_Input()){
      Clock_Delay(100000); // debounce switch
    }
    sw4 = 0;
    Duty = Duty+Change;
    if(Duty >= Period){ // 10 20 30 ... 90%
      Duty = 4;
    }
    Motor_Forward(Duty,Duty);
    DumpI = 0; // Start dump
  }
  SSD1306_SetCursor(7,1);
  // low pass filter
  LeftTach  = ((LPFSIZE-1)*LeftTach  + Tachometer_GetLeftrpm()) /LPFSIZE; 
  RightTach = ((LPFSIZE-1)*RightTach + Tachometer_GetRightrpm())/LPFSIZE;;

  SSD1306_OutUDec(RightTach);
  SSD1306_SetCursor(7,2);
  SSD1306_OutUDec(LeftTach);
  SSD1306_SetCursor(7,3);
  SSD1306_OutUFix1((1000*Duty)/Period);
  SSD1306_SetCursor(7,4);  
  SSD1306_OutUHex7(Bump);  
  lasts4 = sw4;
}
int main7(void){  // main7 to test motors and tachometer
  uint32_t sw4,lasts4;
  DisableInterrupts();
  PLL_Init(Bus80MHz);                   // set system clock to 80 MHz
  Duty = 4;
  Period = 40000;
  Change = 4000;  
  LaunchPad_Init(); // call befor Motor_Init (both access PF3 PF2)
  lasts4 = LaunchPad_Input();
  Bump_Init();   // initialize four bump switches
  SSD1306_Init(SSD1306_SWITCHCAPVCC);
  SSD1306_ClearBuffer();
  SSD1306_OutBuffer();
  SSD1306_SetCursor(0,0);
  SSD1306_OutString("RSLK 2");
  SSD1306_SetCursor(0,1);
  SSD1306_OutString("R rpm =");
  SSD1306_OutUDec(Period); 
  SSD1306_SetCursor(0,2);
  SSD1306_OutString("L rpm =");
  SSD1306_SetCursor(0,3);
  SSD1306_OutString("PWM(%)=");
  SSD1306_SetCursor(0,4);
  SSD1306_OutString("Bump  =");  
  SSD1306_SetCursor(7,1);
  Motor_Init();
  Motor_Forward(Duty,Duty);
//  Motor_Backward(4000,1000);
//  Motor_Right(4000,1000);
//  Motor_Left(4000,1000);
  Tachometer_Init();
  Timer2A_Init(&Background,800000,3); // 10ms

  EnableInterrupts();
  while(1){
    Bump = Bump_In();
    if(Bump){
      Duty = 4;
      Motor_Forward(Duty,Duty);  // stop
      Clock_Delay(100000); // debounce switch
      do{
        Bump = Bump_In();
        SSD1306_SetCursor(7,4);  
        SSD1306_OutUHex7(Bump); 
      }while(Bump);
      Motor_Forward(Duty,Duty);
    }
    sw4 = LaunchPad_Input();
    if(sw4 && (lasts4==0)){ // touch switch
      while(LaunchPad_Input()){
        Clock_Delay(100000); // debounce switch
      }
      sw4 = 0;
      Duty = Duty+Change;
      if(Duty >= Period){ // 10 20 30 ... 90%
        Duty = 4;
      }
      Motor_Forward(Duty,Duty);
      DumpI = 0; // Start dump
    }
    SSD1306_SetCursor(7,1);
    SSD1306_OutUDec(Tachometer_GetRightrpm());
    SSD1306_SetCursor(7,2);
    SSD1306_OutUDec(Tachometer_GetLeftrpm());
    SSD1306_SetCursor(7,3);
    SSD1306_OutUFix1((1000*Duty)/Period);
    SSD1306_SetCursor(7,4);  
    SSD1306_OutUHex7(Bump);  
  }
}


